/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.bestimmungstau.stau;

import java.util.Collection;
import java.util.List;

import de.bsvrz.ibv.anastau.bestimmungstau.indikator.SobMessQuerschnitt;
import de.bsvrz.ibv.anastau.bestimmungstau.indikator.SobStrassenKnoten;
import de.bsvrz.ibv.anastau.bestimmungstau.indikator.SobStrassenTeilSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StrassenSegment;
import de.bsvrz.ibv.anastau.progstau.SvpPrognoseDaten;

/**
 * Interface zum Zugriff auf die Daten eines Stauobjektes.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public interface IStau {
	/**
	 * aktualisiert die für den Stau berechnete Prognoseverlängerung auf Basis
	 * des aktuellen Status der beteiligten Störfallindikatoren.
	 */
	void aktualisierePrognoseVerlaengerung();

	/**
	 * liefert die Anzahl der Fahrstreifen an der angegebenen Position innerhalb
	 * des Staus.
	 *
	 * @param position
	 *            die Position innerhalb des Staus
	 * @return die Anzahl
	 */
	long anzahlFahrStreifen(long position);

	/**
	 * entfernt das Stauobjekt aus der Konfiguration des Datenverteilers und der
	 * Stauobjektverwaltung.
	 */
	void entfernen();

	/**
	 * liefert die Menge der Knoten, die zwischen Stauanfang und dem übergebenen
	 * Messquerschnitt liegen.
	 *
	 * @param mq
	 *            der Messquerschnitt
	 * @return die Menge der ermittelten Abfahrten
	 */
	Collection<SobStrassenKnoten> ermittleAsAmAnfang(MessQuerschnittAllgemein mq);

	/**
	 * liefert die Menge der Knoten, die zwischen Stauende und dem übergebenen
	 * Messquerschnitt liegen.
	 *
	 * @param mq
	 *            der Messquerschnitt
	 * @return die Menge der ermittelten Abfahrten
	 */
	Collection<SobStrassenKnoten> ermittleAsAmEnde(MessQuerschnittAllgemein mq);

	/**
	 * liefert die Menge der Strassenknoten, die sich im Stau befinden.
	 *
	 * @return die Menge der ermittelten Abfahrten
	 */
	Collection<SobStrassenKnoten> getKnotenImStau();

	/**
	 * liefert die reale Länge des Stauobjektes in Metern, ohne die Verlängerung
	 * durch die Prognose.
	 *
	 * @return die Länge
	 */
	long getLaenge();

	/**
	 * liefert das letzte Straßenteilsegment, das sich im Stau befindet. Wenn
	 * kein Straßenteilsegment ermittelt werden konnte, wird der Wert
	 * <code>null</code> geliefert.
	 *
	 * @return das Straßenteilsegment oder <code>null</code>
	 */
	SobStrassenTeilSegment getLetztesStsImStau();

	/**
	 * liefert den ersten Messquerschnitt entgegengesetzt zur Fahrtrichtung
	 * innerhalb des Staus.
	 *
	 * @return den Messquerschnitt oder <code>null</code>, wenn keiner ermittelt
	 *         werden konnte.
	 */
	SobMessQuerschnitt getMqAmAnfang();

	/**
	 * liefert den letzten Messquerschnitt entgegengesetzt zur Fahrtrichtung
	 * innerhalb des Staus.
	 *
	 * @return den Messquerschnitt oder <code>null</code>, wenn keiner ermittelt
	 *         werden konnte.
	 */
	SobMessQuerschnitt getMqAmEnde();

	/**
	 * liefert den ersten Messquerschnitt, der in Fahrtrichtung vor dem Stau und
	 * vor dem nächsten freien Autobahnkreuz, - dreieck oder -ende liegt.
	 *
	 * @return den Messquerschnitt oder <code>null</code>, wenn keiner ermittelt
	 *         werden konnte.
	 */
	SobMessQuerschnitt getMqDanach();

	/**
	 * liefert den ersten Messquerschnitt, der entgegengesetzt zur Fahrtrichtung
	 * noch nicht vom Stau betroffen ist und vor dem nächsten freien
	 * Autobahnkreuz, - dreieck oder -ende liegt.
	 *
	 * @return den Messquerschnitt oder <code>null</code>, wenn keiner ermittelt
	 *         werden konnte.
	 */
	SobMessQuerschnitt getMqDavor();

	/**
	 * liefert die Position des Staus relativ zum Beginn des
	 * Stra&szlig;ensegments in der entsprechenden Richtung.
	 *
	 * @return die Position in Metern
	 */
	long getOffset();

	/**
	 * liefert die Prognosedaten des Stauobjekts. Wenn noch keine Prognose
	 * erfolgte, wird der Wert <code>null</code> geliefert.
	 *
	 * @return die Prognosedaten oder <code>null</code>
	 */
	SvpPrognoseDaten getPrognose();

	/**
	 * liefert den Zeitpunkt, zu dem das Stauobjekt angelegt wurde.<br>
	 * Die Zeitangabe erfolgt in Sekunden seit dem 1.1.1970 00:00:00 GMT.
	 *
	 * @return den Zeitpunkt
	 */
	long getStartZeit();

	/**
	 * liefert den Längenoffset des Stauendes bezüglich der FahrtRichtung der
	 * Straße inklusive der Verlängerung durch die Stauverlaufsprognose.
	 *
	 * @return den Wert
	 */
	double getStrassenEnde();

	/**
	 * liefert die Stra&szlig;ensegmente, auf denen sich der Stau befindet.
	 *
	 * @return die Liste der Stra&szlig;ensegmente
	 */
	List<StrassenSegment> getStrassenSegmente();

	/**
	 * liefert den Anfangsoffset des Staus innerhalb der FahrtRichtung der
	 * Straße.
	 *
	 * @return den Offset
	 */
	double getStrassenStart();

	/**
	 * liefert das erste Straßenteilsegment, das sich in Fahrtrichtung hinter
	 * dem Stau befindet. Wenn kein entsprechendes Straßenteilsegment ermittelt
	 * werden konnte, wird der Wert <code>null</code> geliefert.
	 *
	 * @return das Straßenteilsegment oder <code>null</code>
	 */
	SobStrassenTeilSegment getStsVorDemStau();

	/** publiziert die Prognosedaten eines Staus über den Datenverteiler. */
	void publizierePrognoseDaten();

	/**
	 * setzt die Prognosedaten des Stauobjekts.
	 *
	 * @param prognose
	 *            die Daten
	 */
	void setPrognose(SvpPrognoseDaten prognose);

	/**
	 * überträgt die Daten des Staus an den Datenverteiler.
	 *
	 * @param zeitpunkt
	 *            der Zeitpunkt für den die Daten gesichert werden
	 */
	void sichern(long zeitpunkt);

}
