/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.migration.modell.netz;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.anastau.migration.modell.ObjektFactory;
import de.bsvrz.ibv.anastau.migration.modell.netz.NetzInterface.FahrtRichtung;
import de.bsvrz.ibv.anastau.migration.modell.tmc.zustaende.TmcRichtung;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.AeusseresStrassenSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.Strasse;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StrassenSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StrassenTeilSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StrassenTeilSegment.BetriebsKilometer;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.VerkehrModellNetz;

/**
 * (Singleton-)Klasse zur Umrechnung zwischen den Referenzierungsarten.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class NetzReferenzen {

	/** Das Singleton der Klasse. */
	private static NetzReferenzen singleton;

	/**
	 * Gibt das einzige Objekt der Klasse zur&uuml;ck.
	 *
	 * @return das Singleton der Klasse.
	 */
	public static NetzReferenzen getInstanz() {
		if (singleton == null) {
			singleton = new NetzReferenzen();
		}

		return singleton;
	}

	/** Das Netzmodell. */
	private VerkehrModellNetz netzmodell;

	/**
	 * Initialisiert die Klasse.
	 *
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * @param netz
	 *            Netz
	 * @throws NetzReferenzException
	 *             wenn das Netz nicht initialisiert werden kann
	 */
	public void init(final ClientDavInterface dav, final String netz)
			throws NetzReferenzException {

		if (dav == null) {
			throw new IllegalArgumentException(
					"Parameter 'dav' darf nicht null sein!");
		}

		if (netz == null) {
			throw new IllegalArgumentException(
					"Parameter 'netz' darf nicht null sein!");
		}

		// initialisiere das Netzmodell
		final SystemObject netzObjekt = dav.getDataModel().getObject(netz);
		if (netzObjekt == null) {
			throw new NetzReferenzException(
					"Das Netz '" + netz + "' kann nicht initialisiert werden");
		}
		ObjektFactory.getInstanz().setVerbindung(dav);
		ObjektFactory.getInstanz().registerStandardFactories();

		netzmodell = (VerkehrModellNetz) ObjektFactory.getInstanz()
				.getModellobjekt(netzObjekt);

		// initialisiere das Referenznetz
		NetzReferenzen.getInstanz().setNetzmodell(netzmodell);
	}

	/**
	 * Rechnet Ortsreferenz mit Stra&szlig;e und Betriebskilometer in Angaben
	 * mit Stra&szlig;enSegment und den Offset vom Anfang des
	 * Stra&szlig;enSegments um.
	 *
	 * @param referenz
	 *            Ortsreferenz, bei dem die Ortsangabe über eine Stra&szlig;e und
	 *            den Betriebskilometer dargestellt wird.
	 *
	 * @return Ortsreferenz, bei dem die Ortsangabe &uuml;ber ein
	 *         Stra&szlig;enSegment und den Offset vom Anfang des
	 *         Stra&szlig;enSegments dargestellt wird.
	 *
	 * @throws NetzReferenzException
	 *             wenn keine Abbildung m&ouml;glich ist.
	 */
	public StrassenSegmentUndOffsetOrtsReferenzInterface ermittleOrtsReferenzStrassenSegmentUndOffset(
			final StrasseUndBetriebsKilometerOrtsReferenzInterface referenz)
					throws NetzReferenzException {
		if (referenz == null) {
			throw new NetzReferenzException(
					"Das Argument 'referenz' darf nicht 'null' sein");
		}

		try {
			final ClientDavInterface dav = ObjektFactory.getInstanz()
					.getVerbindung();

			final SystemObject systemObjekt = dav.getDataModel()
					.getObject(referenz.getStrasse().getPid());

			if (systemObjekt == null) {
				throw new NetzReferenzException("Die Strasse '"
						+ referenz.getStrasse().getName()
						+ "' konnte in der Konfiguration nicht gefunden werden");
			}

			final Strasse modellstrasse = (Strasse) ObjektFactory
					.getInstanz().getModellobjekt(systemObjekt);

			if (modellstrasse == null) {
				throw new NetzReferenzException("Die Strasse '"
						+ referenz.getStrasse().getName()
						+ "' konnte in der Konfiguration nicht gefunden werden");
			}

			// entsprechend Absprache ist die Strasse in positiver TMC-Richtung
			// definiert
			final TmcRichtung tmcrichtung = referenz.getFahrtRichtung()
					.equals(NetzInterface.FahrtRichtung.IN_RICHTUNG)
					? TmcRichtung.POSITIV : TmcRichtung.NEGATIV;

			for (final AeusseresStrassenSegment segment : modellstrasse
					.getAuessereStrassensegmente()) {
				// System.out.println("ASS: " + segment.getName());
				if (!segment.getTmcRichtung().equals(tmcrichtung)) {
					continue;
				}

				for (final StrassenTeilSegment teilsegment : segment
						.getStrassenTeilSegmente()) {
					for (final BetriebsKilometer bk : teilsegment
							.getBetriebsKilometer()) {
						if ((Integer.parseInt(bk.getBlockNummer()) == referenz
								.getBlockNummer())
								&& (bk.getWert() == referenz
								.getBetriebsKilometer())) {
							final long offset = (long) (segment
									.getTeilSegmentOffset(teilsegment)
									+ bk.getOffset());
							return new StrassenSegmentUndOffsetOrtsReferenz(
									segment.getPid(), offset);
						}
					}
				}
			}
		} catch (final Exception e) {
			throw new NetzReferenzException(
					"Keine Abbildung möglich: " + e.getMessage());
		}

		throw new NetzReferenzException("Keine Abbildung möglich");
	}

	/**
	 * Rechnet Ortsreferenz mit Stra&szlig;enSegment und Offset vom Anfang des
	 * Stra&szlig;enSegments in Angaben &uuml;ber eine Straße und den
	 * Betriebskilometers um.
	 *
	 * @param referenz
	 *            Ortsreferenz, bei dem die Ortsangabe &uuml;ber ein
	 *            Stra&szlig;enSegment und den Offset vom Anfang des
	 *            Stra&szlig;enSegments dargestellt wird.
	 *
	 * @return Ortsreferenz, bei dem die Ortsangabe über eine Stra&szlig;e und
	 *         den Betriebskilometer dargestellt wird.
	 *
	 * @throws NetzReferenzException
	 *             wenn keine Abbildung m&ouml;glich ist.
	 */
	public StrasseUndBetriebsKilometerOrtsReferenzInterface ermittleOrtsReferenzStrasseUndBetriebsKilometer(
			final StrassenSegmentUndOffsetOrtsReferenzInterface referenz)
					throws NetzReferenzException {
		if (referenz == null) {
			throw new NetzReferenzException(
					"Das Argument 'referenz' darf nicht 'null' sein");
		}

		try {
			final StrassenSegment segment = ((StrassenSegmentUndOffsetOrtsReferenz) referenz)
					.getModelSegment();
			// if (segment instanceof InneresStrassenSegment) {
			// throw new NetzReferenzException(
			// "Keine Abbildung möglich da InneresStraßenSegment");
			// }

			// bestimme die Straße
			final Strasse strasse = new Strasse(
					((StrassenSegmentUndOffsetOrtsReferenz) referenz)
					.getModelSegment().getStrasse().getSystemObject());

			// bestimme die Richtung
			final FahrtRichtung fahrtRichtung;
			final TmcRichtung segmentTmcRictung = ((AeusseresStrassenSegment) segment)
					.getTmcRichtung();
			// entsprechend Absprache ist die Strasse in positiver TMC-Richtung
			// definiert
			if (segmentTmcRictung == TmcRichtung.POSITIV) {
				fahrtRichtung = NetzInterface.FahrtRichtung.IN_RICHTUNG;
			} else if (segmentTmcRictung == TmcRichtung.NEGATIV) {
				fahrtRichtung = NetzInterface.FahrtRichtung.GEGEN_RICHTUNG;
			} else {
				throw new NetzReferenzException(
						"Die Richtung der Straße kann nicht bestimmt werden");
			}

			final SegmentBetriebsKilometer sbk = new SegmentBetriebsKilometer(
					strasse, fahrtRichtung, segment);

			return sbk
					.findeBetriebsKilometerAmOffset(referenz.getStartOffset());
		} catch (final Exception e) {
			throw new NetzReferenzException(
					"Keine Abbildung möglich: " + e.getMessage());
		}

	}

	public String getAsbKnotenKey(final String knoten) {
		return knoten.substring(0, 7);
	}

	/**
	 * Test, ob 2 Netzknoten &uuml;berenstimmen. Ntzknoten ohne Kennung
	 * (Buchstaben am Ende) werden gleich der 'Ohne-Kennung' 'O' behandelt.
	 *
	 * @param knoten1
	 *            Netzknoten
	 * @param knoten2
	 *            Netzknoten
	 * @return true, wenn gleich
	 */
	public boolean netzKnotenMatch(final String knoten1, final String knoten2) {
		return knoten1.equals(knoten2) || (isOhneKennung(knoten1)
				&& isOhneKennung(knoten2)
				&& getAsbKnotenKey(knoten1).equals(getAsbKnotenKey(knoten2)));
	}

	private boolean isOhneKennung(final String knoten) {
		if (knoten.length() == 7) {
			return true;
		}

		final String kennung = knoten.substring(7, 8);

		return "O".equalsIgnoreCase(kennung);
	}

	/**
	 * ordnet der Klasse das Netmodell zu.
	 *
	 * @param netzmodell
	 *            das Netzmodell
	 *
	 */
	public void setNetzmodell(final VerkehrModellNetz netzmodell) {
		this.netzmodell = netzmodell;
	}
}
