/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.progstau.parameter;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.VerkehrModellNetz;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Verwaltung und Bereitstellung der Parameter für die
 * Stauverlaufsprognose.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SvpParameterObserver implements ISvpParameterObserver, ClientReceiverInterface {

	/**
	 * Logger für Debugausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * die Attributgruppe zur Bestimmung der Parameter der Stauverlaufsprognose.
	 */
	private final AttributeGroup parameterAtg;

	/**
	 * der aktuelle Datensatz mit den Parametern für die Stauverlaufsprognose.
	 */
	private Data parameterSet;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt eine Instanz der Parameterverwaltungsklasse für das
	 * übergebene Netz auf der Basis der angegebenen Datenverteilerverbindung.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @param netz
	 *            das Netz, an dem die Parameter definiert sind
	 */
	public SvpParameterObserver(final ClientDavInterface dav, final VerkehrModellNetz netz) {
		final DataModel model = dav.getDataModel();

		parameterAtg = model.getAttributeGroup(SvpParameterDefs.ATG_NAME_STAUPROGNOSE_MODELL);
		final Aspect parameterAsp = model.getAspect("asp.parameterSoll");
		dav.subscribeReceiver(this, netz.getSystemObject(), new DataDescription(parameterAtg, parameterAsp),
				ReceiveOptions.normal(), ReceiverRole.receiver());
	}

	@Override
	public final long getDaempfungszeit() {
		long result = SvpParameterDefs.SVP_DEFAULT_DAEMPFUNGSZEIT;
		if (parameterSet != null) {
			result = parameterSet.getTimeValue(SvpParameterDefs.ATT_DAEMPFUNGSZEIT).getMillis();
		}
		return result;
	}

	@Override
	public final long getFaktorQ0() {
		long result = SvpParameterDefs.SVP_DEFAULT_FAKTOR_Q0;
		if (parameterSet != null) {
			result = parameterSet.getScaledValue(SvpParameterDefs.ATT_FAKTOR_Q0).longValue();
		}
		return result;
	}

	@Override
	public final long getHorizont() {
		long result = SvpParameterDefs.SVP_DEFAULT_HORIZONT;
		if (parameterSet != null) {
			result = parameterSet.getTimeValue(SvpParameterDefs.ATT_HORIZONT).getMillis();
		}
		return result;
	}

	@Override
	public final long getIterationen() {
		long result = SvpParameterDefs.SVP_DEFAULT_ITERATIONEN;
		if (parameterSet != null) {
			result = parameterSet.getUnscaledValue(SvpParameterDefs.ATT_ITERATIONEN).longValue();
		}
		return Math.max(1, result);
	}

	@Override
	public final long getLaengePkw() {
		long result = SvpParameterDefs.SVP_DEFAULT_LAENGE_PKW;
		if (parameterSet != null) {
			result = parameterSet.getUnscaledValue(SvpParameterDefs.ATT_LAENGE_PKW).longValue();
		}
		return result;
	}

	@Override
	public final long getSchrittWeite() {
		return (long) Math.ceil((double) getHorizont() / getIterationen());
	}

	@Override
	public void update(final ResultData[] results) {
		for (final ResultData result : results) {
			if (result.getDataDescription().getAttributeGroup().equals(parameterAtg)) {
				if (result.hasData()) {
					parameterSet = result.getData();
				} else {
					String message = "Keine Daten verfügbar";
					if (result.isNoSourceAvailable()) {
						message += ": Keine Quelle";
					} else if (result.isNoDataAvailable()) {
						message += ": Keine Daten";
					} else if (result.isNoRightsAvailable()) {
						message += ": Keine Rechte";
					}
					SvpParameterObserver.LOGGER.info(message);
				}
			}
		}
	}
}
